<?php
/**
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * @copyright 2005 Camptocamp SA
 * @package CorePlugins
 * @author Yves Bolognini <yves.bolognini@camptocamp.com>
 * @version $Id: ServerTables.php,v 1.8 2005-10-04 17:25:52 asaunier Exp $
 */

/**
 * Table rules management
 */
require_once(CARTOWEB_HOME . 'coreplugins/tables/common/TableRulesRegistry.php');

/**
 * Server side of Tables plugin
 *
 * Provides table rules registry to server plugins.
 * @package CorePlugins
 */
class ServerTables extends ServerPlugin {

    /**
     * @var Logger
     */
    private $log;

    /**
     * Registry which contains all client side rules
     * @var TableRulesRegistry
     */ 
    protected $tableRulesRegistry = null;

    /**
     * Constructor
     */
    public function __construct() {
        $this->log =& LoggerManager::getLogger(__CLASS__);
        parent::__construct();
    }
    
    /**
     * Returns current table rules registry
     * @return TableRulesRegistry
     */
    public function getTableRulesRegistry() {
        if (is_null($this->tableRulesRegistry)) {
            $this->tableRulesRegistry =  new TableRulesRegistry();
        }
        return $this->tableRulesRegistry;
    }
    
    /**
     * Applies rules on tables
     * @param array
     * @return array array of {@link TableGroup}
     */
    public function applyRules($tables) {
        return $this->getTableRulesRegistry()->applyRules($tables);
    }    
}

?>
